import pika
import sys
import json
from datetime import datetime
from flowcept.flowceptor.plugins.abstract_flowceptor import AbstractFlowceptor


class ZambezeInterceptor(AbstractFlowceptor):
    def __init__(self, plugin_key):
        super().__init__(plugin_key)

    def intercept(self, message: dict):
        now = datetime.now()
        dt_string = now.strftime("%d/%m/%Y %H:%M:%S")
        # TODO: make constants
        intercepted_message = {"time": dt_string, "application_msg": {}}
        for key in self.settings.keys_to_intercept:
            if key in message:
                intercepted_message["application_msg"][key] = message[key]

        super().post_intercept(intercepted_message)

    def observe(self):
        connection = pika.BlockingConnection(
            pika.ConnectionParameters(
                host=self.settings.host, port=self.settings.port
            )
        )
        channel = connection.channel()
        channel.queue_declare(queue=self.settings.queue_name)
        channel.basic_consume(
            queue=self.settings.queue_name,
            on_message_callback=self.callback,
            auto_ack=True,
        )

        print(" [*] Waiting for Zambeze messages. To exit press CTRL+C")
        channel.start_consuming()

    def callback(self, ch, method, properties, body):
        # TODO: consider making it a superclass method
        """
        function that decides what do to when a change is identified.
        If it's an interesting change, it calls self.intercept; otherwise,
        let it go....
        """
        body_obj = json.loads(body)

        for key_value in self.settings.key_values_to_filter:
            if key_value.key in body_obj:
                if body_obj[key_value.key] == key_value.value:
                    print(
                        f"I'm an interceptor and I need to intercept this:"
                        f"\n\t{json.dumps(body_obj)}"
                    )
                    self.intercept(message=body_obj)
                    break


if __name__ == "__main__":
    try:
        # TODO: allow passing the interceptor key in the argv
        interceptor = ZambezeInterceptor("zambeze1")
        interceptor.observe()
    except KeyboardInterrupt:
        print("Interrupted")
        sys.exit(0)



import unittest
import threading
import time

from flowcept.flowcept_consumer.consumer import consume_intercepted_messages
from flowcept.flowceptor.plugins.mlflow.mlflow_interceptor import (
    MLFlowInterceptor,
)


class TestMLFlow(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestMLFlow, self).__init__(*args, **kwargs)
        self.interceptor = MLFlowInterceptor("mlflow1")

        threading.Thread(target=self.interceptor.observe, daemon=True).start()
        threading.Thread(
            target=consume_intercepted_messages, daemon=True
        ).start()
        time.sleep(3)

    def test_mlflow(self):
        import uuid
        import mlflow

        # from mlflow.tracking import MlflowClient
        # client = MlflowClient()
        mlflow.set_tracking_uri(
            f"sqlite:///" f"{self.interceptor.settings.file_path}"
        )
        experiment_name = "LinearRegression"
        experiment_id = mlflow.create_experiment(
            experiment_name + str(uuid.uuid4())
        )
        with mlflow.start_run(experiment_id=experiment_id):
            mlflow.log_params({"number_epochs": 10})
            print("\nTrained model")
            mlflow.log_metric("loss", 0.04)

        time.sleep(3)


if __name__ == "__main__":
    unittest.main()



from watchdog.events import LoggingEventHandler


class InterceptionEventHandler(LoggingEventHandler):
    def __init__(self, interceptor_instance, callback_function):
        super().__init__()
        self.callback_function = callback_function
        self.interceptor_instance = interceptor_instance

    def on_modified(self, event):
        self.callback_function(self.interceptor_instance)



import sys
import redis
from flowcept.configs import REDIS_PORT, REDIS_HOST, REDIS_CHANNEL


def send_to_service(message):
    # TODO: implement
    print(message)
    pass


def consume_intercepted_messages():
    r = redis.Redis(host=REDIS_HOST, port=REDIS_PORT, db=0)
    p = r.pubsub()
    p.psubscribe(REDIS_CHANNEL)
    print(f"Subscribed to {REDIS_CHANNEL}. Waiting for messages.")
    for message in p.listen():
        if message["type"] in {"psubscribe"}:  # TODO make it a constant
            continue
        print(
            f"I'm a Flowceptor consumer and I received this message:"
            f""
            f"\n\t{message}"
        )
        send_to_service(message)


if __name__ == "__main__":
    try:
        consume_intercepted_messages()
    except KeyboardInterrupt:
        print("Interrupted")
        sys.exit(0)



import os

PROJECT_DIR_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "..")
)

_settings_path = os.path.join(PROJECT_DIR_PATH, "resources", "settings.yaml")
SETTINGS_PATH = os.getenv("SETTINGS_PATH", _settings_path)

REDIS_HOST = os.getenv("REDIS_HOST", "localhost")
REDIS_PORT = int(os.getenv("REDIS_PORT", "6379"))
REDIS_CHANNEL = "interception"



from flask import Flask, request, jsonify

app = Flask(__name__)


@app.route("/")
def liveness():
    return "Server up!"


@app.route("/interception", methods=["POST"])
def interception():
    data = request.get_json()
    print(data)
    return jsonify(data), 200


if __name__ == "__main__":
    app.run()



import unittest
import json
import threading
import pika

from flowcept.flowcept_consumer.consumer import consume_intercepted_messages

from flowcept.flowceptor.plugins.zambeze.zambeze_interceptor import (
    ZambezeInterceptor,
)
from flowcept.flowceptor.plugins.zambeze.zambeze_message import ZambezeMessage


class TestZambeze(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestZambeze, self).__init__(*args, **kwargs)
        self.interceptor = ZambezeInterceptor("zambeze1")

        self._connection = pika.BlockingConnection(
            pika.ConnectionParameters(
                self.interceptor.settings.host, self.interceptor.settings.port
            )
        )
        self._channel = self._connection.channel()
        self._channel.queue_declare(queue=self.interceptor.settings.queue_name)
        threading.Thread(target=self.interceptor.observe, daemon=True).start()
        threading.Thread(
            target=consume_intercepted_messages, daemon=True
        ).start()

    def test_send_message(self):
        msg = ZambezeMessage(
            **{
                "name": "ImageMagick",
                "activity_id": "xyz-uuid",
                "campaign_id": "abc-uuid",
                "origin_agent_id": "def-uuid",
                "files": ["globus://Users/6o1/file.txt"],
                "command": "convert",
                "activity_status": "CREATED",
                "arguments": [
                    "-delay",
                    "20",
                    "-loop",
                    "0",
                    "~/tests/campaigns/imagesequence/*.jpg",
                    "a.gif",
                ],
                "kwargs": {},
                "depends_on": [],
            }
        )

        self._channel.basic_publish(
            exchange="",
            routing_key=self.interceptor.settings.queue_name,
            body=json.dumps(msg.__dict__),
        )

        print(" [x] Sent msg")
        self._connection.close()
        import time

        time.sleep(3)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Any, List


@dataclass
class KeyValuesToFilter:
    key: str
    value: Any


@dataclass
class ZambezeSettings:

    host: str
    port: int
    queue_name: str
    key_values_to_filter: List[KeyValuesToFilter]
    keys_to_intercept: List[str]
    kind: str = "zambeze"
    observer_type: str = "message_broker"
    observer_subtype: str = "rabbit_mq"


@dataclass
class MLFlowSettings:

    file_path: str
    log_params: List[str]
    log_metrics: List[str]
    watch_interval_sec: int
    kind: str = "mlflow"
    observer_type: str = "db"
    observer_subtype: str = "sqlite"



from abc import ABCMeta, abstractmethod
import os
import json
import yaml
import redis

from flowcept.commons.vocabulary import Vocabulary
from flowcept.configs import (
    PROJECT_DIR_PATH,
    SETTINGS_PATH,
    REDIS_HOST,
    REDIS_PORT,
    REDIS_CHANNEL,
)
from flowcept.flowceptor.plugins.settings_data_classes import (
    ZambezeSettings,
    KeyValuesToFilter,
    MLFlowSettings,
)


class AbstractFlowceptor(object, metaclass=ABCMeta):
    def __init__(self, plugin_key):
        self.settings = AbstractFlowceptor.__get_settings(plugin_key)
        self._redis = redis.Redis(host=REDIS_HOST, port=REDIS_PORT, db=0)

    @staticmethod
    def __get_settings(plugin_key):
        # TODO: use factory pattern
        with open(SETTINGS_PATH) as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
        settings = data[Vocabulary.Settings.PLUGINS][plugin_key]
        if (
            settings[Vocabulary.Settings.KIND]
            == Vocabulary.Settings.ZAMBEZE_KIND
        ):
            settings_obj: ZambezeSettings = ZambezeSettings(**settings)
            settings_obj.key_values_to_filter = [
                KeyValuesToFilter(**item)
                for item in settings_obj.key_values_to_filter
            ]
            return settings_obj
        elif (
            settings[Vocabulary.Settings.KIND]
            == Vocabulary.Settings.MLFLOW_KIND
        ):
            settings_obj: MLFlowSettings = MLFlowSettings(**settings)
            if not os.path.isabs(settings_obj.file_path):
                settings_obj.file_path = os.path.join(
                    PROJECT_DIR_PATH, settings_obj.file_path
                )
            return settings_obj

    @abstractmethod
    def intercept(self, message: dict):
        raise NotImplementedError()

    @abstractmethod
    def observe(self):
        raise NotImplementedError()

    def post_intercept(self, intercepted_message: dict):
        print(
            f"Going to send to Redis an intercepted message:"
            f"\n\t{json.dumps(intercepted_message)}"
        )
        self._redis.publish(REDIS_CHANNEL, json.dumps(intercepted_message))



import sys
import os
import time
from watchdog.observers import Observer
from flowcept.flowceptor.plugins.abstract_flowceptor import AbstractFlowceptor

from flowcept.flowceptor.plugins.mlflow.interception_event_handler import (
    InterceptionEventHandler,
)


class MLFlowInterceptor(AbstractFlowceptor):
    def intercept(self, message: dict):
        super().post_intercept(message)

    @staticmethod
    def callback(interceptor_instance: "MLFlowInterceptor"):
        """
        function that decides what do to when a change is identified.
        If it's an interesting change, it calls self.intercept; otherwise,
        let it go....
        """
        # TODO get latest info
        interceptor_instance.intercept({"nothing": "yet"})

    def observe(self):
        event_handler = InterceptionEventHandler(
            self, MLFlowInterceptor.callback
        )
        while not os.path.isfile(self.settings.file_path):
            print(
                f"I can't watch the file {self.settings.file_path},"
                f" as it does not exist."
            )
            print(
                f"\tI will sleep for {self.settings.watch_interval_sec} sec."
                f" to see if it appears."
            )
            time.sleep(self.settings.watch_interval_sec)

        observer = Observer()
        observer.schedule(
            event_handler, self.settings.file_path, recursive=True
        )
        observer.start()
        print(f"Watching {self.settings.file_path}")


if __name__ == "__main__":
    try:
        interceptor = MLFlowInterceptor("mlflow1")
        interceptor.observe()
        while True:
            time.sleep(interceptor.settings.watch_interval_sec)
    except KeyboardInterrupt:
        print("Interrupted")
        sys.exit(0)



from dataclasses import dataclass
from typing import List, Dict


@dataclass
class ZambezeMessage:

    name: str
    activity_id: str
    campaign_id: str
    origin_agent_id: str
    files: List[str]
    command: str
    activity_status: str
    arguments: List[str]
    kwargs: Dict
    depends_on: List[str]



from setuptools import setup, find_packages

from flowcept.version import __version__

with open("README.md") as fh:
    long_description = fh.read()

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

with open("extra_requirements/zambeze-requirements.txt") as f:
    zambeze_plugin_requirements = f.read().splitlines()

with open("extra_requirements/mlflow-requirements.txt") as f:
    mlflow_plugin_requirements = f.read().splitlines()

full_requirements = (
    requirements + zambeze_plugin_requirements + mlflow_plugin_requirements
)

setup(
    name="flowcept",
    version=__version__,
    license="MIT",
    author="Oak Ridge National Laboratory",
    author_email="support@flowcept.org",
    description="A tool to intercept dataflows",  # TODO: change later
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ORNL/flowcept",
    include_package_data=True,
    install_requires=requirements,
    extras_require={
        "full": full_requirements,
        "mlflow": mlflow_plugin_requirements,
        "zambeze": zambeze_plugin_requirements,
    },
    packages=find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Topic :: Documentation :: Sphinx",
        "Topic :: System :: Distributed Computing",
    ],
    python_requires=">=3.9",  # TODO: try a lower version
    # scripts=["bin/flowcept"],
)
